import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import { formatToBase64 } from './photoTools';
import { DEFAULT_HAS_ERRORS_STATUS, DEFAULT_IMAGE_ROTATION_DEGREES, REJECT_FILES_REASONS } from './config';
export var filterValidFiles = function filterValidFiles(_ref) {
  var files = _ref.files,
      filesToBeFiltered = _ref.filesToBeFiltered,
      acceptedFileMaxSize = _ref.acceptedFileMaxSize,
      handlePhotosRejected = _ref.handlePhotosRejected,
      setMaxSizeError = _ref.setMaxSizeError,
      setMaxPhotosError = _ref.setMaxPhotosError,
      allowUploadDuplicatedPhotos = _ref.allowUploadDuplicatedPhotos,
      maxPhotos = _ref.maxPhotos;
  var notExcedingMaxSizeFiles = [];
  var excedingMaxSizeFiles = [];

  var notExcedingMaxSizeFilesFilter = function notExcedingMaxSizeFilesFilter(filesToFilter) {
    return filesToFilter.map(function (fileToFilter) {
      if (fileToFilter.size >= acceptedFileMaxSize) {
        excedingMaxSizeFiles.push({
          rejectedFile: fileToFilter,
          reason: "" + REJECT_FILES_REASONS.maxSize + acceptedFileMaxSize
        });
      } else {
        notExcedingMaxSizeFiles.push(fileToFilter);
      }
    });
  };

  var notRepeatedFiles = [];
  var repeatedFiles = [];

  var notRepeatedFilesFilter = function notRepeatedFilesFilter(filesToFilter) {
    return filesToFilter.map(function (fileToFilter) {
      var newFilePath = fileToFilter.path,
          newFileSize = fileToFilter.size,
          newFileLastModified = fileToFilter.lastModified;
      var isFileAlready = files.some(function (file) {
        var url = file.url,
            properties = file.properties;
        if (url) return false;
        var path = properties.path,
            size = properties.size,
            lastModified = properties.lastModified;
        return path === newFilePath && size === newFileSize && lastModified === newFileLastModified;
      });

      if (isFileAlready) {
        repeatedFiles.push({
          rejectedFile: fileToFilter,
          reason: REJECT_FILES_REASONS.repeated
        });
      } else {
        notRepeatedFiles.push(fileToFilter);
      }
    });
  };

  notExcedingMaxSizeFilesFilter(filesToBeFiltered);

  if (excedingMaxSizeFiles.length) {
    setMaxSizeError();
    handlePhotosRejected(excedingMaxSizeFiles);
  }

  if (allowUploadDuplicatedPhotos) {
    notRepeatedFiles.push.apply(notRepeatedFiles, notExcedingMaxSizeFiles);
  } else {
    notRepeatedFilesFilter(notExcedingMaxSizeFiles);
  }

  if (repeatedFiles.length) {
    handlePhotosRejected(repeatedFiles);
  }

  if (!notRepeatedFiles.length) return notRepeatedFiles;

  if (files.length + notRepeatedFiles.length >= maxPhotos) {
    setMaxPhotosError();
    var howManyFilesToMax = maxPhotos - files.length;
    notRepeatedFiles.splice(howManyFilesToMax - 1, notRepeatedFiles.length - howManyFilesToMax);
  }

  return notRepeatedFiles;
};
export function callbackUploadPhotoHandler(_x, _x2, _x3) {
  return _callbackUploadPhotoHandler.apply(this, arguments);
}

function _callbackUploadPhotoHandler() {
  _callbackUploadPhotoHandler = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(blob, callbackUploadPhoto, oldUrl) {
    var response;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (!callbackUploadPhoto) {
              _context2.next = 10;
              break;
            }

            _context2.prev = 1;
            _context2.next = 4;
            return callbackUploadPhoto(blob, oldUrl);

          case 4:
            response = _context2.sent;
            return _context2.abrupt("return", response.url);

          case 8:
            _context2.prev = 8;
            _context2.t0 = _context2["catch"](1);

          case 10:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, null, [[1, 8]]);
  }));
  return _callbackUploadPhotoHandler.apply(this, arguments);
}

export var prepareFiles = function prepareFiles(_ref2) {
  var callbackUploadPhoto = _ref2.callbackUploadPhoto,
      currentFiles = _ref2.currentFiles,
      defaultFormatToBase64Options = _ref2.defaultFormatToBase64Options,
      errorCorruptedPhotoUploadedText = _ref2.errorCorruptedPhotoUploadedText,
      handlePhotosRejected = _ref2.handlePhotosRejected,
      newFiles = _ref2.newFiles,
      setCorruptedFileError = _ref2.setCorruptedFileError,
      setFiles = _ref2.setFiles,
      setIsLoading = _ref2.setIsLoading,
      _scrollToBottom = _ref2._scrollToBottom,
      _callbackPhotosUploaded = _ref2._callbackPhotosUploaded;
  newFiles.reduce(function (accumulatorPromise, nextFile, index) {
    return accumulatorPromise.then(function () {
      return formatToBase64({
        file: nextFile,
        options: defaultFormatToBase64Options
      });
    }).then( /*#__PURE__*/function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(_ref3) {
        var file, blob, originalBase64, croppedBase64, rotation, _ref3$hasErrors, hasErrors, errorText, url;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                file = _ref3.file, blob = _ref3.blob, originalBase64 = _ref3.originalBase64, croppedBase64 = _ref3.croppedBase64, rotation = _ref3.rotation, _ref3$hasErrors = _ref3.hasErrors, hasErrors = _ref3$hasErrors === void 0 ? DEFAULT_HAS_ERRORS_STATUS : _ref3$hasErrors;

                if (!hasErrors) {
                  _context.next = 7;
                  break;
                }

                errorText = errorCorruptedPhotoUploadedText.replace('%{filepath}', nextFile.path);
                handlePhotosRejected([{
                  rejectedFile: nextFile,
                  reason: REJECT_FILES_REASONS.loadFailed
                }]);
                setCorruptedFileError(errorText);
                _context.next = 11;
                break;

              case 7:
                _context.next = 9;
                return callbackUploadPhotoHandler(blob, callbackUploadPhoto);

              case 9:
                url = _context.sent;
                currentFiles.push({
                  blob: blob,
                  file: file,
                  hasErrors: hasErrors,
                  isModified: false,
                  isNew: true,
                  originalBase64: originalBase64,
                  properties: {
                    path: nextFile.path,
                    size: nextFile.size,
                    lastModified: nextFile.lastModified
                  },
                  preview: croppedBase64,
                  rotation: rotation,
                  url: url
                });

              case 11:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x4) {
        return _ref4.apply(this, arguments);
      };
    }()).then(function () {
      setFiles([].concat(currentFiles));

      if (index >= newFiles.length - 1) {
        setIsLoading(false);

        _scrollToBottom();

        _callbackPhotosUploaded(currentFiles);
      }
    });
  }, Promise.resolve());
};
export var loadInitialPhotos = function loadInitialPhotos(_ref5) {
  var initialPhotos = _ref5.initialPhotos,
      defaultFormatToBase64Options = _ref5.defaultFormatToBase64Options,
      setInitialDownloadError = _ref5.setInitialDownloadError,
      setFiles = _ref5.setFiles,
      _callbackPhotosUploaded = _ref5._callbackPhotosUploaded,
      setIsLoading = _ref5.setIsLoading;
  var filesWithBase64 = initialPhotos.map(function (url) {
    return formatToBase64({
      url: url,
      options: defaultFormatToBase64Options
    });
  });
  Promise.all(filesWithBase64).then(function (newFiles) {
    var readyPhotos = newFiles.map(function (_ref6) {
      var blob = _ref6.blob,
          croppedBase64 = _ref6.croppedBase64,
          url = _ref6.url,
          _ref6$hasErrors = _ref6.hasErrors,
          hasErrors = _ref6$hasErrors === void 0 ? DEFAULT_HAS_ERRORS_STATUS : _ref6$hasErrors;
      return {
        blob: blob,
        url: url,
        hasErrors: hasErrors,
        originalBase64: croppedBase64,
        preview: croppedBase64,
        rotation: DEFAULT_IMAGE_ROTATION_DEGREES,
        isNew: false,
        isModified: false
      };
    });

    if (readyPhotos.some(function (photos) {
      return photos.hasErrors;
    })) {
      setInitialDownloadError();
    }

    setFiles([].concat(readyPhotos));

    _callbackPhotosUploaded(readyPhotos);

    setIsLoading(false);
  });
};